@@ -8,6 +8,7 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
+import textwrap
 from random import Random
 
 import pytest
@@ -18,11 +19,14 @@ from hypothesis.internal.conjecture.data import ConjectureData, Status, StopTest
 from hypothesis.internal.conjecture.datatree import (
     Branch,
     DataTree,
+    PreviouslyUnseenBehaviour,
     compute_max_children,
 )
 from hypothesis.internal.conjecture.engine import ConjectureRunner
 from hypothesis.internal.conjecture.floats import float_to_int
+from hypothesis.internal.escalation import InterestingOrigin
 from hypothesis.internal.floats import next_up
+from hypothesis.vendor import pretty
 
 from tests.conjecture.common import (
     draw_boolean_kwargs,
@@ -31,6 +35,7 @@ from tests.conjecture.common import (
     draw_integer_kwargs,
     draw_string_kwargs,
     fresh_data,
+    ir_nodes,
     run_to_buffer,
 )
 
@@ -519,16 +524,13 @@ def test_can_generate_hard_values():
         data.draw_integer(min_value, max_value)
         data.freeze()
 
-    @run_to_buffer
-    def expected_buf(data):
-        data.draw_integer(min_value, max_value, forced=max_value)
-        data.mark_interesting()
-
     # this test doubles as conjecture coverage for using our child cache, so
     # ensure we don't miss that logic by getting lucky and drawing the correct
     # value once or twice.
     for _ in range(20):
-        assert tree.generate_novel_prefix(Random()) == expected_buf
+        prefix = tree.generate_novel_prefix(Random())
+        data = ConjectureData.for_buffer(prefix)
+        assert data.draw_integer(min_value, max_value) == 1000
 
 
 def test_can_generate_hard_floats():
@@ -570,3 +572,174 @@ def test_can_generate_hard_floats():
         prefix = tree.generate_novel_prefix(Random())
         data = ConjectureData.for_buffer(prefix)
         assert data.draw_float(min_value, max_value, allow_nan=False) == expected_value
+
+
+@given(draw_boolean_kwargs(), draw_integer_kwargs())
+def test_datatree_repr(bool_kwargs, int_kwargs):
+    tree = DataTree()
+
+    try:
+        int("not an int")
+    except ValueError as e:
+        origin = InterestingOrigin.from_exception(e)
+
+    observer = tree.new_observer()
+    observer.draw_boolean(True, was_forced=False, kwargs=bool_kwargs)
+    observer.conclude_test(Status.INVALID, interesting_origin=None)
+
+    observer = tree.new_observer()
+    observer.draw_boolean(False, was_forced=False, kwargs=bool_kwargs)
+    observer.draw_integer(42, was_forced=False, kwargs=int_kwargs)
+    observer.conclude_test(Status.VALID, interesting_origin=None)
+
+    observer = tree.new_observer()
+    observer.draw_boolean(False, was_forced=False, kwargs=bool_kwargs)
+    observer.draw_integer(0, was_forced=False, kwargs=int_kwargs)
+    observer.draw_boolean(False, was_forced=True, kwargs=bool_kwargs)
+    observer.conclude_test(Status.INTERESTING, interesting_origin=origin)
+
+    observer = tree.new_observer()
+    observer.draw_boolean(False, was_forced=False, kwargs=bool_kwargs)
+    observer.draw_integer(5, was_forced=False, kwargs=int_kwargs)
+
+    assert (
+        pretty.pretty(tree)
+        == textwrap.dedent(
+            f"""
+        boolean True {bool_kwargs}
+          Conclusion (Status.INVALID)
+        boolean False {bool_kwargs}
+          integer 42 {int_kwargs}
+            Conclusion (Status.VALID)
+          integer 0 {int_kwargs}
+            boolean False [forced] {bool_kwargs}
+              Conclusion (Status.INTERESTING, {origin})
+          integer 5 {int_kwargs}
+            unknown
+        """
+        ).strip()
+    )
+
+
+def _draw(data, node, *, forced=None):
+    return getattr(data, f"draw_{node.ir_type}")(**node.kwargs, forced=forced)
+
+
+@given(ir_nodes(), ir_nodes())
+@settings(suppress_health_check=[HealthCheck.too_slow])
+def test_misaligned_nodes_after_valid_draw(node, misaligned_node):
+    # if we run a valid tree through a test function, the datatree should still
+    # be able to return a Status.INVALID when a node in that tree becomes misaligned.
+    assume(misaligned_node.ir_type != node.ir_type)
+    tree = DataTree()
+
+    data = ConjectureData.for_ir_tree([node], observer=tree.new_observer())
+    _draw(data, node)
+    assert data.status is Status.VALID
+
+    data = ConjectureData.for_ir_tree([misaligned_node])
+    tree.simulate_test_function(data)
+    assert data.status is Status.INVALID
+
+    assert data.invalid_at == (node.ir_type, node.kwargs, None)
+
+
+@given(ir_nodes(was_forced=False), ir_nodes(was_forced=False))
+@settings(suppress_health_check=[HealthCheck.too_slow])
+def test_misaligned_nodes_before_valid_draw(node, misaligned_node):
+    # if we run a misaligned tree through a test function, we should still get
+    # the correct response when running the aligned version of the tree through
+    # the test function afterwards.
+    assume(misaligned_node.ir_type != node.ir_type)
+    tree = DataTree()
+
+    data = ConjectureData.for_ir_tree([node], observer=tree.new_observer())
+
+    with pytest.raises(StopTest):
+        _draw(data, misaligned_node)
+    data.freeze()
+    assert data.status is Status.INVALID
+    assert data.examples.ir_tree_nodes == []
+
+    # make sure the tree is tracking that `node` leads to Status.INVALID only
+    # when trying to draw a misaligned node. If we try to draw something that
+    # is valid for that node, then it's a valid draw and should lead to Status.VALID.
+    data = ConjectureData.for_ir_tree([node], observer=tree.new_observer())
+    _draw(data, node)
+    data.freeze()
+    assert data.status is Status.VALID
+    assert data.examples.ir_tree_nodes == [node]
+
+
+@given(ir_nodes(was_forced=True, ir_type="float"))
+def test_simulate_forced_floats(node):
+    tree = DataTree()
+
+    data = ConjectureData.for_ir_tree([node], observer=tree.new_observer())
+    data.draw_float(**node.kwargs, forced=node.value)
+    with pytest.raises(StopTest):
+        data.conclude_test(Status.VALID)
+
+    data = ConjectureData.for_ir_tree([node], observer=tree.new_observer())
+    tree.simulate_test_function(data)
+    data.freeze()
+    assert data.examples.ir_tree_nodes == [node]
+
+
+@given(ir_nodes(), ir_nodes())
+@settings(suppress_health_check=[HealthCheck.too_slow])
+def test_simulate_non_invalid_conclude_is_unseen_behavior(node, misaligned_node):
+    # coverage test for the case where we have a node invalid at some draw, and
+    # simulating that draw should lead to some conclusion other than invalid.
+    assume(misaligned_node.ir_type != node.ir_type)
+    tree = DataTree()
+
+    # set up an invalid draw. the transition for node is now None in the tree
+    data = ConjectureData.for_ir_tree(
+        [node, misaligned_node], observer=tree.new_observer()
+    )
+    with pytest.raises(StopTest):
+        _draw(data, node)
+        _draw(data, node)
+
+    # try simulating something that will take the transition path for node, but
+    # end in something other than a misaligned/invalid result, such as an overrun
+    data = ConjectureData.for_ir_tree([node], observer=tree.new_observer())
+    with pytest.raises(PreviouslyUnseenBehaviour):
+        tree.simulate_test_function(data)
+
+    assert data.status is Status.OVERRUN
+
+
+@given(ir_nodes(), ir_nodes())
+@settings(suppress_health_check=[HealthCheck.too_slow])
+def test_simulating_inherits_invalid_forced_status(node, misaligned_node):
+    assume(misaligned_node.ir_type != node.ir_type)
+
+    # we have some logic in DataTree.simulate_test_function to "peek ahead" and
+    # make sure it simulates invalid nodes correctly. But if it does so without
+    # respecting whether the invalid node was forced or not, and this simulation
+    # is observed by an observer, this can cause flaky errors later due to a node
+    # going from unforced to forced.
+
+    tree = DataTree()
+
+    def test_function(ir_nodes):
+        data = ConjectureData.for_ir_tree(ir_nodes, observer=tree.new_observer())
+        _draw(data, node)
+        _draw(data, node, forced=node.value)
+
+    # (1) set up a misaligned node at index 1
+    with pytest.raises(StopTest):
+        test_function([node, misaligned_node])
+
+    # (2) simulate an aligned tree. the datatree peeks ahead here using invalid_at
+    # due to (1).
+    data = ConjectureData.for_ir_tree([node, node], observer=tree.new_observer())
+    with pytest.raises(PreviouslyUnseenBehaviour):
+        tree.simulate_test_function(data)
+
+    # (3) run the same aligned tree without simulating. this uses the actual test
+    # function's draw and forced value. This would flaky error if it did not match
+    # what the datatree peeked ahead with in (2).
+    test_function([node, node])
