@@ -8,6 +8,7 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
+import enum
 import re
 import time
 from random import Random
@@ -15,12 +16,21 @@ from unittest.mock import Mock
 
 import pytest
 
-from hypothesis import HealthCheck, Phase, Verbosity, settings
+from hypothesis import (
+    HealthCheck,
+    Phase,
+    Verbosity,
+    assume,
+    given,
+    settings,
+    strategies as st,
+)
 from hypothesis.database import ExampleDatabase, InMemoryExampleDatabase
 from hypothesis.errors import FailedHealthCheck, Flaky
-from hypothesis.internal.compat import int_from_bytes
+from hypothesis.internal.compat import bit_count, int_from_bytes
 from hypothesis.internal.conjecture import engine as engine_module
-from hypothesis.internal.conjecture.data import ConjectureData, Overrun, Status
+from hypothesis.internal.conjecture.data import ConjectureData, IRNode, Overrun, Status
+from hypothesis.internal.conjecture.datatree import compute_max_children
 from hypothesis.internal.conjecture.engine import (
     MIN_TEST_CALLS,
     ConjectureRunner,
@@ -29,17 +39,18 @@ from hypothesis.internal.conjecture.engine import (
     RunIsComplete,
 )
 from hypothesis.internal.conjecture.pareto import DominanceRelation, dominance
-from hypothesis.internal.conjecture.shrinker import Shrinker, block_program
+from hypothesis.internal.conjecture.shrinker import Shrinker
 from hypothesis.internal.entropy import deterministic_PRNG
 
+from tests.common.debug import minimal
 from tests.common.strategies import SLOW, HardToShrink
 from tests.common.utils import no_shrink
 from tests.conjecture.common import (
     SOME_LABEL,
     TEST_SETTINGS,
     buffer_size_limit,
+    ir_nodes,
     run_to_buffer,
-    run_to_data,
     shrinking_from,
 )
 
@@ -440,17 +451,46 @@ def test_fails_health_check_for_slow_draws():
 def test_can_shrink_variable_draws(n_large):
     target = 128 * n_large
 
-    @run_to_data
-    def data(data):
-        n = data.draw_integer(0, 15)
-        b = [data.draw_integer(0, 255) for _ in range(n)]
-        if sum(b) >= target:
-            data.mark_interesting()
+    @st.composite
+    def strategy(draw):
+        n = draw(st.integers(0, 15))
+        return [draw(st.integers(0, 255)) for _ in range(n)]
 
-    x = data.buffer
+    ints = minimal(strategy(), lambda ints: sum(ints) >= target)
+    # should look like [4, 255, 255, 255]
+    assert ints == [target % 255] + [255] * (len(ints) - 1)
 
-    assert x.count(0) == 0
-    assert sum(x[1:]) == target
+
+def test_can_shrink_variable_string_draws():
+    @st.composite
+    def strategy(draw):
+        n = draw(st.integers(min_value=0, max_value=20))
+        return draw(st.text(st.characters(codec="ascii"), min_size=n, max_size=n))
+
+    s = minimal(strategy(), lambda s: len(s) >= 10 and "a" in s)
+
+    # TODO_BETTER_SHRINK: this should be
+    # assert s == "0" * 9 + "a"
+    # but we first shrink to having a single a at the end of the string and then
+    # fail to apply our special case invalid logic when shrinking the min_size n,
+    # because that logic removes from the end of the string (which fails our
+    # precondition).
+    assert re.match("0+a", s)
+
+
+def test_variable_size_string_increasing():
+    # coverage test for min_size increasing during shrinking (because the test
+    # function inverts n).
+    @st.composite
+    def strategy(draw):
+        n = 10 - draw(st.integers(0, 10))
+        return draw(st.text(st.characters(codec="ascii"), min_size=n, max_size=n))
+
+    s = minimal(strategy(), lambda s: len(s) >= 5 and "a" in s)
+    # TODO_BETTER_SHRINK for the same reason as test_can_shrink_variable_string_draws.
+    # should be
+    # assert s == "0000a"
+    assert re.match("0+a", s)
 
 
 def test_run_nothing():
@@ -472,7 +512,7 @@ def test_debug_data(capsys):
 
     def f(data):
         for x in bytes(buf):
-            if data.draw_integer(0, 2**8 - 1) != x:
+            if data.draw(st.integers(0, 100)) != x:
                 data.mark_invalid()
             data.start_example(1)
             data.stop_example()
@@ -828,7 +868,7 @@ def test_dependent_block_pairs_can_lower_to_zero():
         if n == 1:
             data.mark_interesting()
 
-    shrinker.fixate_shrink_passes(["minimize_individual_blocks"])
+    shrinker.fixate_shrink_passes(["minimize_individual_nodes"])
     assert list(shrinker.shrink_target.buffer) == [0, 1]
 
 
@@ -841,7 +881,7 @@ def test_handle_size_too_large_during_dependent_lowering():
         else:
             data.draw_integer(0, 2**8 - 1)
 
-    shrinker.fixate_shrink_passes(["minimize_individual_blocks"])
+    shrinker.fixate_shrink_passes(["minimize_individual_nodes"])
 
 
 def test_block_may_grow_during_lexical_shrinking():
@@ -857,11 +897,11 @@ def test_block_may_grow_during_lexical_shrinking():
             data.draw_integer(0, 2**16 - 1)
         data.mark_interesting()
 
-    shrinker.fixate_shrink_passes(["minimize_individual_blocks"])
+    shrinker.fixate_shrink_passes(["minimize_individual_nodes"])
     assert list(shrinker.shrink_target.buffer) == [0, 0, 0]
 
 
-def test_lower_common_block_offset_does_nothing_when_changed_blocks_are_zero():
+def test_lower_common_node_offset_does_nothing_when_changed_blocks_are_zero():
     @shrinking_from([1, 0, 1, 0])
     def shrinker(data):
         data.draw_boolean()
@@ -872,11 +912,11 @@ def test_lower_common_block_offset_does_nothing_when_changed_blocks_are_zero():
 
     shrinker.mark_changed(1)
     shrinker.mark_changed(3)
-    shrinker.lower_common_block_offset()
+    shrinker.lower_common_node_offset()
     assert list(shrinker.shrink_target.buffer) == [1, 0, 1, 0]
 
 
-def test_lower_common_block_offset_ignores_zeros():
+def test_lower_common_node_offset_ignores_zeros():
     @shrinking_from([2, 2, 0])
     def shrinker(data):
         n = data.draw_integer(0, 2**8 - 1)
@@ -887,26 +927,10 @@ def test_lower_common_block_offset_ignores_zeros():
 
     for i in range(3):
         shrinker.mark_changed(i)
-    shrinker.lower_common_block_offset()
+    shrinker.lower_common_node_offset()
     assert list(shrinker.shrink_target.buffer) == [1, 1, 0]
 
 
-def test_pandas_hack():
-    @shrinking_from([2, 1, 1, 7])
-    def shrinker(data):
-        n = data.draw_integer(0, 2**8 - 1)
-        m = data.draw_integer(0, 2**8 - 1)
-        if n == 1:
-            if m == 7:
-                data.mark_interesting()
-        data.draw_integer(0, 2**8 - 1)
-        if data.draw_integer(0, 2**8 - 1) == 7:
-            data.mark_interesting()
-
-    shrinker.fixate_shrink_passes([block_program("-XX")])
-    assert list(shrinker.shrink_target.buffer) == [1, 7]
-
-
 def test_cached_test_function_returns_right_value():
     count = [0]
 
@@ -1230,10 +1254,11 @@ def test_populates_the_pareto_front():
         assert len(runner.pareto_front) == 2**4
 
 
-def test_pareto_front_contains_smallest_valid_when_not_targeting():
+def test_pareto_front_contains_smallest_valid():
     with deterministic_PRNG():
 
         def test(data):
+            data.target_observations[""] = 1
             data.draw_integer(0, 2**4 - 1)
 
         runner = ConjectureRunner(
@@ -1251,55 +1276,6 @@ def test_pareto_front_contains_smallest_valid_when_not_targeting():
         assert len(runner.pareto_front) == 1
 
 
-def test_pareto_front_contains_different_interesting_reasons():
-    with deterministic_PRNG():
-
-        def test(data):
-            data.mark_interesting(data.draw_integer(0, 2**4 - 1))
-
-        runner = ConjectureRunner(
-            test,
-            settings=settings(
-                max_examples=5000,
-                database=InMemoryExampleDatabase(),
-                suppress_health_check=list(HealthCheck),
-            ),
-            database_key=b"stuff",
-        )
-
-        runner.run()
-
-        assert len(runner.pareto_front) == 2**4
-
-
-def test_clears_defunct_pareto_front():
-    with deterministic_PRNG():
-
-        def test(data):
-            data.draw_integer(0, 2**8 - 1)
-            data.draw_integer(0, 2**8 - 1)
-
-        db = InMemoryExampleDatabase()
-
-        runner = ConjectureRunner(
-            test,
-            settings=settings(
-                max_examples=10000,
-                database=db,
-                suppress_health_check=list(HealthCheck),
-                phases=[Phase.reuse],
-            ),
-            database_key=b"stuff",
-        )
-
-        for i in range(256):
-            db.save(runner.pareto_key, bytes([i, 0]))
-
-        runner.run()
-
-        assert len(list(db.fetch(runner.pareto_key))) == 1
-
-
 def test_replaces_all_dominated():
     def test(data):
         data.target_observations["m"] = 3 - data.draw_integer(0, 3)
@@ -1574,3 +1550,151 @@ def test_too_slow_report():
     got = state.timing_report()
     print(got)
     assert expected == got
+
+
+def _draw(cd, node):
+    return getattr(cd, f"draw_{node.ir_type}")(**node.kwargs)
+
+
+@given(st.data())
+# drawing a second node with a different ir_type is hard to satisfy, as hypothesis
+# biases towards the first defined ir_type early on.
+@settings(suppress_health_check=[HealthCheck.too_slow, HealthCheck.filter_too_much])
+def test_extensions_of_misaligned_trees_are_cached(data):
+    # ConjectureData treats all ir node prefixes as if they were not forced,
+    # so was_forced doesn't make a difference when drawing. In fact, was_forced
+    # will always be False after running a tree through ConjectureData, so comparing
+    # for equality before and after is easiest if was_forced is always False.
+    node = data.draw(ir_nodes(was_forced=False))
+    misaligned_node = data.draw(ir_nodes(was_forced=False))
+    assume(node.ir_type != misaligned_node.ir_type)
+    # avoid trivial nodes resulting in exhausting the tree extremely early
+    assume(compute_max_children(node.ir_type, node.kwargs) > 100)
+
+    def test(cd):
+        _draw(cd, node)
+        _draw(cd, node)
+
+    runner = ConjectureRunner(test)
+
+    def _assert_cached(cd):
+        assert runner.call_count == 1
+        assert cd.status is Status.INVALID
+        assert cd.examples.ir_tree_nodes == [node]
+
+    assert runner.call_count == 0
+
+    cd = runner.cached_test_function_ir([node, misaligned_node])
+    _assert_cached(cd)
+
+    cd = runner.cached_test_function_ir([node, misaligned_node])
+    _assert_cached(cd)
+
+    extension = data.draw(st.lists(ir_nodes(was_forced=False)))
+    cd = runner.cached_test_function_ir([node, misaligned_node, *extension])
+    _assert_cached(cd)
+
+
+@given(ir_nodes(was_forced=False), ir_nodes(was_forced=False))
+@settings(suppress_health_check=[HealthCheck.too_slow, HealthCheck.filter_too_much])
+def test_misaligned_tree_does_not_clobber_cache(node, misaligned_node):
+    assume(node.ir_type != misaligned_node.ir_type)
+    assume(compute_max_children(node.ir_type, node.kwargs) > 100)
+
+    def test(cd):
+        _draw(cd, node)
+
+    runner = ConjectureRunner(test)
+    assert runner.call_count == 0
+
+    data_buffer = runner.cached_test_function(b"")
+    assert runner.call_count == 1
+    assert data_buffer.status is Status.OVERRUN
+
+    data_ir = runner.cached_test_function_ir([misaligned_node])
+    assert data_ir.status is Status.INVALID
+    assert data_ir.buffer == b""
+    assert runner.call_count == 2
+
+    result = runner.cached_test_function(b"")
+    assert runner.call_count == 2
+    assert result == data_buffer
+
+
+@pytest.mark.parametrize("forced_first", [True, False])
+@given(node=ir_nodes(was_forced=False))
+def test_cache_ignores_was_forced(forced_first, node):
+    assume(compute_max_children(node.ir_type, node.kwargs) > 100)
+    forced_node = IRNode(
+        ir_type=node.ir_type,
+        value=node.value,
+        kwargs=node.kwargs,
+        was_forced=True,
+    )
+
+    def test(cd):
+        _draw(cd, node)
+
+    runner = ConjectureRunner(test)
+    assert runner.call_count == 0
+
+    runner.cached_test_function_ir([forced_node] if forced_first else [node])
+    assert runner.call_count == 1
+
+    runner.cached_test_function_ir([node] if forced_first else [forced_node])
+    assert runner.call_count == 1
+
+
+def test_simulate_to_evicted_data(monkeypatch):
+    # test that we do not rely on the false invariant that correctly simulating
+    # a data to a result means we have that result in the cache, due to e.g.
+    # cache evictions (but also potentially other trickery).
+    monkeypatch.setattr(engine_module, "CACHE_SIZE", 1)
+
+    node_0 = IRNode(
+        ir_type="integer",
+        value=0,
+        kwargs={
+            "min_value": None,
+            "max_value": None,
+            "weights": None,
+            "shrink_towards": 0,
+        },
+        was_forced=False,
+    )
+    node_1 = node_0.copy(with_value=1)
+
+    def test(data):
+        data.draw_integer()
+
+    runner = ConjectureRunner(test)
+    runner.cached_test_function_ir([node_0])
+    # cache size is 1 so this evicts node_0
+    runner.cached_test_function_ir([node_1])
+    assert runner.call_count == 2
+
+    # we dont throw PreviouslyUnseenBehavior when simulating, but the result
+    # was evicted to the cache so we will still call through to the test function.
+    runner.tree.simulate_test_function(ConjectureData.for_ir_tree([node_0]))
+    runner.cached_test_function_ir([node_0])
+    assert runner.call_count == 3
+
+
+@pytest.mark.parametrize(
+    "strategy, condition",
+    [
+        (st.lists(st.integers(), min_size=5), lambda v: True),
+        (st.lists(st.text(), min_size=2, unique=True), lambda v: True),
+        (
+            st.sampled_from(
+                enum.Flag("LargeFlag", {f"bit{i}": enum.auto() for i in range(64)})
+            ),
+            lambda f: bit_count(f.value) > 1,
+        ),
+    ],
+)
+def test_mildly_complicated_strategies(strategy, condition):
+    # There are some code paths in engine.py and shrinker.py that are easily
+    # covered by shrinking any mildly compliated strategy and aren't worth
+    # testing explicitly for. This covers those.
+    minimal(strategy, condition)
