@@ -13,7 +13,7 @@ from hypothesis.database import InMemoryExampleDatabase
 from hypothesis.internal.compat import int_from_bytes
 from hypothesis.internal.conjecture.data import ConjectureData
 from hypothesis.internal.conjecture.engine import ConjectureRunner
-from hypothesis.internal.conjecture.shrinker import Shrinker, block_program
+from hypothesis.internal.conjecture.shrinker import Shrinker, node_program
 
 from tests.common.utils import counts_calls, non_covering_examples
 from tests.conjecture.common import run_to_buffer, shrinking_from
@@ -123,7 +123,7 @@ def test_cached_with_masked_byte_agrees_with_results(byte_a, byte_b):
     assert (cached_a is cached_b) == (cached_a.buffer == data_b.buffer)
 
 
-def test_block_programs_fail_efficiently(monkeypatch):
+def test_node_programs_fail_efficiently(monkeypatch):
     # Create 256 byte-sized blocks. None of the blocks can be deleted, and
     # every deletion attempt produces a different buffer.
     @shrinking_from(bytes(range(256)))
@@ -136,12 +136,12 @@ def test_block_programs_fail_efficiently(monkeypatch):
             data.mark_interesting()
 
     monkeypatch.setattr(
-        Shrinker, "run_block_program", counts_calls(Shrinker.run_block_program)
+        Shrinker, "run_node_program", counts_calls(Shrinker.run_node_program)
     )
 
     shrinker.max_stall = 500
 
-    shrinker.fixate_shrink_passes([block_program("XX")])
+    shrinker.fixate_shrink_passes([node_program("XX")])
 
     assert shrinker.shrinks == 0
     assert 250 <= shrinker.calls <= 260
@@ -150,4 +150,4 @@ def test_block_programs_fail_efficiently(monkeypatch):
     # bit of wiggle room for implementation details.
     #   - Too many calls mean that failing steps are doing too much work.
     #   - Too few calls mean that this test is probably miscounting and buggy.
-    assert 250 <= Shrinker.run_block_program.calls <= 260
+    assert 250 <= Shrinker.run_node_program.calls <= 260
