@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import pytest
 
@@ -19,6 +14,8 @@ from hypothesis import find, given, strategies as st
 from hypothesis.errors import InvalidArgument
 from hypothesis.strategies._internal.core import defines_strategy
 
+from tests.common.debug import check_can_generate_examples
+
 
 def test_does_not_error_on_initial_calculation():
     st.floats(max_value=float("nan"))
@@ -30,9 +27,9 @@ def test_does_not_error_on_initial_calculation():
 def test_errors_each_time():
     s = st.integers(max_value=1, min_value=3)
     with pytest.raises(InvalidArgument):
-        s.example()
+        check_can_generate_examples(s)
     with pytest.raises(InvalidArgument):
-        s.example()
+        check_can_generate_examples(s)
 
 
 def test_errors_on_test_invocation():
@@ -53,7 +50,7 @@ def test_errors_on_find():
 def test_errors_on_example():
     s = st.floats(min_value=2.0, max_value=1.0)
     with pytest.raises(InvalidArgument):
-        s.example()
+        check_can_generate_examples(s)
 
 
 def test_does_not_recalculate_the_strategy():
@@ -66,7 +63,7 @@ def test_does_not_recalculate_the_strategy():
 
     f = foo()
     assert calls == [0]
-    f.example()
+    check_can_generate_examples(f)
     assert calls == [1]
-    f.example()
+    check_can_generate_examples(f)
     assert calls == [1]
